module WorkTimer

class HtmlExportWriter

public

   def export(filename, theTimeClock, startTime, endTime)
      actDoc = REXML::Document.new
      actDoc.add_element("html")
            
      eventHash = theTimeClock.createEventHash(startTime, endTime)
      
      #Divide the time interval into days
      numDays = (endTime - startTime) / (3600 * 24)
            
      if (endTime - startTime) % (3600 * 24) > 0
         numDays += 1
      end
      
      #create an array to hold the totals
      dailyTotals = Array.new(numDays,0)
      
      #create a hash to hold the totals for each
      #activity type
      activityTotals = Hash.new
      
      startTimeString = Time.at(startTime).strftime("%m/%d/%Y")
      
      # note: we're actually producing a report for the interval
      # [startTime, endTime) so we should subtract a second from
      # the end time string so that we don't lead the user to
      # think that the report includes the next day
      endTimeString = Time.at(endTime-1).strftime("%m/%d/%Y")
      
      # create the head element with the appropriate document title
      headElement = REXML::Element.new("head", actDoc.root)
      titleElement = REXML::Element.new("title", headElement)
      titleElement.text = 
         "Activity Report (#{startTimeString} - #{endTimeString})"
      
      # create a body element and add a h1 title 
      bodyElement = REXML::Element.new("body", actDoc.root)
      reportTitleElement = REXML::Element.new("h1", bodyElement)
      reportTitleElement.add_attribute("align", "center")
      reportTitleElement.text = 
         "Activity Report (#{startTimeString} - #{endTimeString})"
      
      # create the table element
      tableElement = REXML::Element.new("table", bodyElement)
      tableElement.add_attribute("border", 1)
      tableElement.add_attribute("align", "center")
      
      # create the date column header
      headerRow1 = REXML::Element.new("tr", tableElement)
      dateColumnHeader = REXML::Element.new("td", headerRow1)
      dateColumnHeader.add_attribute("rowspan", 2)
      dateColumnHeader.add_attribute("align", "center")
      dateColumnHeader.text = "Date"
            
      # create the activity totals column header
      activityTotalsColumnHeader = REXML::Element.new("td", headerRow1)
      activityTotalsColumnHeader.add_attribute("colspan", eventHash.size)
      activityTotalsColumnHeader.add_attribute("align", "center")
      activityTotalsColumnHeader.text = "Activity Totals (hrs)"
      
      # create the daily total header
      dailyTotalColumnHeader = REXML::Element.new("td", headerRow1)
      dailyTotalColumnHeader.add_attribute("rowspan", 2)
      dailyTotalColumnHeader.add_attribute("align", "center")
      dailyTotalColumnHeader.text = "Daily Total (hrs)"
            
      # now add the activity name headers
      headerRow2 = REXML::Element.new("tr", tableElement)
      
      eventHash.each_key {
         |desc|
         activityTotals[desc] = 0
         activityHeader = REXML::Element.new("td", headerRow2)
         activityHeader.add_attribute("align", "center")
         activityHeader.text = desc
      }
                        
      #get a summary of activities for each day
      1.upto(numDays) {
         |dayCount|
         dayStart = startTime + (dayCount - 1) * 3600 * 24
         dayEnd = dayStart + 3600 * 24
         
         # create a new row for this date
         dateRowElement = REXML::Element.new("tr", tableElement)
         dateDataElement = REXML::Element.new("td", dateRowElement)
         dateDataElement.text = Time.at(dayStart).strftime("%m/%d/%Y")
         
         # now, for each activity, go through and add the appropriate
         # data for this row                                            
         eventHash.each_pair {
            |description, eventContainer|
                        
            #get the amount of time spent in hours
            totalEventTime = eventContainer.computeTimeSpent(dayStart, dayEnd)
            
            totalHrs      = totalEventTime / 3600.0
            
            dailyTotals[dayCount - 1] += totalHrs
            
            activityTotals[description] += totalHrs
            
            # create the daily total for this activity
            activityDataElement = REXML::Element.new("td", dateRowElement)
            activityDataElement.text = sprintf("%.2f", totalHrs)
            activityDataElement.add_attribute("align", "center")
         }  
         
         # now add the value for the daily total
         dailyTotalElement = REXML::Element.new("td", dateRowElement)
         dailyTotalElement.text = sprintf("%.2f", dailyTotals[dayCount-1])
         dailyTotalElement.add_attribute("align", "center")
      }
      
      # add a blank row
      REXML::Element.new("tr", tableElement)
      
      # now add a row for the activity totals
      activityTotalRowElement = REXML::Element.new("tr", tableElement)
      activityTotalRowHeaderElement = REXML::Element.new("td", activityTotalRowElement)
      activityTotalRowHeaderElement.add_attribute("align", "right")
      italicElement = REXML::Element.new("i", activityTotalRowHeaderElement)
      italicElement.text = "Total:"
      
      grandTotalHours = 0
      
      # now, let's iterate over the activity totals
      activityTotals.each_pair {
         |desc, value|
         grandTotalHours += value
         
         activityTotalDataElement = REXML::Element.new("td", activityTotalRowElement)
         activityTotalDataElement.text = sprintf("%.2f", value)
         activityTotalDataElement.add_attribute("align", "center")
      }
      
      # now add a final data point for the grandTotal
      grandTotalDataElement = REXML::Element.new("td", activityTotalRowElement)
      grandTotalDataElement.text = sprintf("%.2f", grandTotalHours)
      grandTotalDataElement.add_attribute("align", "center")
      
      theFile = File.open(filename, "w")
      
      #write the document
      actDoc.write(theFile,0)
      
      theFile.close
   
   end
end

end